//#ifdef __GNUG__
//#pragma implementation
//#endif
//#include "wx/wxprec.h"

//#ifndef  WX_PRECOMP
  #include "wx/wx.h"
  #include <wx/listctrl.h>
//#endif //precompiled headers


#include "main.h"
#include "main_frame.h"
#include "algorithm.h"
#include "alg_try.h"
#include "video.h"
#include "image_scope.h"
#include "image_object.h"
#include "plugin.h"
#include <dlfcn.h>

IMPLEMENT_APP(PixalcoApp)

const int DEFAULT_IMAGE_BUFFER_SIZE = 5;
const int DEFAULT_MOVIE_BUFFER_SIZE = 10;

PixalcoApp::PixalcoApp()
{
    mImageBufferSize = DEFAULT_IMAGE_BUFFER_SIZE;
    mMovieBufferSize = DEFAULT_MOVIE_BUFFER_SIZE;
    mMainFrame = NULL;
    VideoManager::Create();
    
    // TODO: needs to support each line and row
    // when that is supported, this has to be moved
    for ( int i=0; i<MAX_ROWS; ++i )
    {
        mImageObjBuffer[i] = new list<ImageObject*>();
    }
}

PixalcoApp::~PixalcoApp()
{
    ClearMovieBuffer();
    for ( int i=0; i<MAX_ROWS; ++i )
    {
        ClearBuffer(0, i);
        delete mImageObjBuffer[i];
    }
       
    RemoveAllAlgorithms();
    RemoveAllMediaFiles();
    VideoManager::Destroy();
}

bool PixalcoApp::OnInit()
{
    ::wxInitAllImageHandlers();
    
	mMainFrame = new MainFrame(wxT("Pixalco"), 50, 50, 450, 300);

    Algorithm * alg = new AlgTry1();
    AddAlgorithm(alg);
    alg = new AlgTry2();
    AddAlgorithm(alg);
    alg = new AlgTry3();
    AddAlgorithm(alg);
    alg = new RGBtoBlackWhiteAvg();
    AddAlgorithm(alg);
    alg = new RGBtoBlackWhiteNTSC();
    AddAlgorithm(alg);
    alg = new MovieDiffImage();
    AddAlgorithm(alg);

    mMainFrame->Show(TRUE);
    SetTopWindow(mMainFrame);
    return TRUE;	
}

void PixalcoApp::AddMediaFile(const string& displayName_, const string& fileName_)
{
    wxListCtrl &listImages = mMainFrame->AccessImagesList();
        
    string name(displayName_);
	long id=listImages.FindItem(-1, wxString(name.c_str(), wxConvUTF8) );
    while ( id >= 0 )
    {
	    name += "_";
	    id = listImages.FindItem(-1, wxString(name.c_str(), wxConvUTF8));
    }

    ImageObject * imageObj = new ImageObject(name);
    
	if (    imageObj->GetImage()->CanRead( wxString(fileName_.c_str(), wxConvUTF8) )
	        && imageObj->GetImage()->LoadFile( wxString(fileName_.c_str(), wxConvUTF8) ) )
    {
        MediaFile media;
        media.mMediaType = MEDIA_IMAGE;
        media.mFileName = fileName_;
        media.mDisplayName = displayName_;
        media.mImageObject = imageObj;
        mMediaFiles.push_back(media);
	    listImages.InsertItem( listImages.GetItemCount(), wxString(name.c_str(), wxConvUTF8) );
    }
    else // video support
    {
        Video * video = new Video();
	    if ( video->LoadFile(fileName_.c_str()) )
        {
            int width, height;
            video->GetSize(width, height);
            imageObj->GetImage()->Create(width, height);
            MediaFile media;
            media.mMediaType = MEDIA_MOVIE;
            media.mFileName = fileName_;
            media.mDisplayName = displayName_;
            media.mVideo = video;
            media.mImageObject = imageObj;
            mMediaFiles.push_back(media);
	        listImages.InsertItem( listImages.GetItemCount(), wxString(name.c_str(), wxConvUTF8) );
        }
        else
        {
            imageObj->Release();
            delete video;
        }
    }
}

MediaFile * PixalcoApp::GetMediaFile(const string& displayName_)
{
    for (vector<MediaFile>::iterator it = mMediaFiles.begin(); it != mMediaFiles.end(); it++)
    {
        if ( it->mDisplayName == displayName_ )
        {
            return &(*it);
        }
    }
    return NULL;
}

void PixalcoApp::RemoveMediaFile(const string& displayName_)
{
    for (vector<MediaFile>::iterator it = mMediaFiles.begin(); it != mMediaFiles.end(); it++)
    {
        if ( it->mDisplayName == displayName_ )
        {
            wxListCtrl &listImages = mMainFrame->AccessImagesList();
	        long id=listImages.FindItem(-1, wxString(displayName_.c_str(), wxConvUTF8));
            if ( id >= 0 )
                listImages.DeleteItem(id);
            
            delete it->mVideo;
            if ( it->mImageObject )
                it->mImageObject->Release();
            mMediaFiles.erase(it);
            
            return;
        }
    }
}

void PixalcoApp::SelectMediaFile(const string& displayName_)
{
    MediaFile * info = GetMediaFile(displayName_);
    SetMediaFile(info);
}

// TODO: think about movie stepping
void PixalcoApp::SetMediaFile(MediaFile * media_)
{
    if ( !media_ )
        return;
    
    ImageObject * old = mMainFrame->GetImageScope(0,0)->GetImageObject();        
    if ( media_->GetMediaType() ==  MEDIA_MOVIE )
    {

        media_->mVideo->GetNextFrame( media_->mImageObject->GetImage() );
        
        ImageObject * copy = media_->mImageObject->Copy();
        AddImageObjToMovieBuffer(copy);
        copy->Release();

        mMainFrame->UpdateMovieScrollerPos(media_->mVideo);
    }
    else if ( old /*&& old != media_->mImageObject*/ )
    {
        AddImageObjToBuffer(old, 0, 0);
    }

    ImageObject * copy = media_->mImageObject->Copy();
    mMainFrame->GetImageScope(0,0)->SetImageObject(copy);
    copy->Release();
}

void PixalcoApp::RemoveAllMediaFiles(bool updateMainFrame_)
{
    wxListCtrl &listImages = mMainFrame->AccessImagesList();
    for (vector<MediaFile>::iterator it = mMediaFiles.begin(); it != mMediaFiles.end(); it++)
    {
        if ( updateMainFrame_ )
        {
	        long id=listImages.FindItem(-1, wxString(it->mDisplayName.c_str(), wxConvUTF8));
            if ( id >= 0 )
                listImages.DeleteItem(id);
        }
            
         delete it->mVideo;
         if ( it->mImageObject )
            it->mImageObject->Release();
    }
    mMediaFiles.clear();
}

void PixalcoApp::RunMovie(const string& displayName_, const string& algorithmName_)
{
    // TODO
}

void PixalcoApp::StopMovie(const string& displayName_)
{
    // TODO
}

// TODO: maybe - think about movie stepping
void PixalcoApp::StepMovie(const string& displayName_, const string& algorithmName_)
{
    MediaFile * media = GetMediaFile(displayName_);
    if ( !media )
        return;

    SetMediaFile(media);
    PIXALCO.ExecuteAlgorithm(algorithmName_);
}

void PixalcoApp::SeekMovie(const string& displayName_, long pos_)
{
    MediaFile * media = PIXALCO.GetMediaFile(displayName_);

    if ( media && media->GetMediaType() == MEDIA_MOVIE )
    {
        media->mVideo->Seek(pos_);
        media->mVideo->GetNextFrame( media->mImageObject->GetImage() );
        mMainFrame->UpdateMovieScrollerPos(media->mVideo);
        mMainFrame->GetImageScope(0,0)->SetImageObject(media->mImageObject);
    }
}

void PixalcoApp::AddAlgorithm(Algorithm *algorithm_)
{
    if ( !algorithm_ )
        return;
    
    wxListCtrl &listAlgorithms = mMainFrame->AccessAlgorithmsList();
	listAlgorithms.InsertItem( 0, wxString(algorithm_->GetName().c_str(), wxConvUTF8) );
    
    mAlgorithms.push_back(algorithm_);
}

Algorithm* PixalcoApp::GetAlgorithm(const string& name_)
{
    for(vector<Algorithm*>::iterator it = mAlgorithms.begin(); it != mAlgorithms.end(); it++ )
    {
        if ( (*it)->GetName() == name_ )
        {
            return (*it);
        }
    }
    return NULL;
}

void PixalcoApp::RemoveAlgorithm(const string& name_)
{
    for(vector<Algorithm*>::iterator it = mAlgorithms.begin(); it != mAlgorithms.end(); it++ )
    {
        if ( (*it)->GetName() == name_ )
        {
            wxListCtrl &listAlgorithms = mMainFrame->AccessAlgorithmsList();
	        long id=listAlgorithms.FindItem(-1, wxString(name_.c_str(), wxConvUTF8));
            if ( id >= 0 )
                listAlgorithms.DeleteItem(id);
                        
            delete (*it);
            mAlgorithms.erase(it);
            return;
        }
    }
}

void PixalcoApp::RemoveAllAlgorithms()
{
    for(vector<Algorithm*>::iterator it = mAlgorithms.begin(); it != mAlgorithms.end(); it++ )
        delete *it;
    mAlgorithms.clear();
}

void PixalcoApp::ExecuteAlgorithm(const string& name_)
{
	Algorithm * alg = GetAlgorithm( name_.c_str() );
    if ( !alg )
    {
        if ( name_.length() )
            fprintf(stderr, "PixalcoApp::ExecuteAlgorithm: algorithm not found");
        return;
    }

    int line = 0;
    for ( int i=0; i<3; i++ )
    {
        mCopies[i] = NULL;
    }

    alg->Execute();

    for ( int i=0; i<3; i++ )
    {
        if ( mCopies[i] )
        {
            AddImageObjToBuffer(mCopies[i], 0, i);
            mCopies[i]->Release();
            mCopies[i] = NULL;

            ImageScope * scope = mMainFrame->GetImageScope(line, i);
            if ( scope )
                scope->Refresh(false);
        }
    }
}

void PixalcoApp::AddImageObjToBuffer(ImageObject * imgObj_, int line_, int row_)
{
    if ( !imgObj_ || row_ < 0 || row_ >= MAX_ROWS || line_ < 0 || line_ > 0 )
        return;

//    cerr << imgObj_->GetDisplayName();

    imgObj_->AddRef();
    mImageObjBuffer[row_]->push_front(imgObj_);
    if ( mImageObjBuffer[row_]->size() > mImageBufferSize )
    {
        mImageObjBuffer[row_]->back()->Release();
        mImageObjBuffer[row_]->pop_back();
    } 
}

ImageObject* PixalcoApp::GetImageObjFromBuffer(int line_, int row_, int age_)
{
    if ( row_ < 0 || row_ >= MAX_ROWS || line_ < 0 || line_ > 0 )
        return NULL;

    if ( mImageObjBuffer[row_]->size() <= age_ )
        return NULL;

    list<ImageObject*>::iterator it=mImageObjBuffer[row_]->begin();
    for ( int i=0; i < age_; ++it, ++i);
    
    return (*it);
}

// age_ == 0 is current, otherwise from buffer
wxImage* PixalcoApp::GetImage(int row_, int age_) 
{
    // TODO: line should depend on line in which current algorithm runs
    int line = 0;

    if ( age_ < 0 )
        return NULL;

    if ( age_ == 0 )
    {
        ImageScope * scope = mMainFrame->GetImageScope(line, row_);
        if ( !scope || !scope->GetImageObject())
            return NULL;
        mCopies[row_] = scope->GetImageObject()->Copy();
        return scope->GetImage();
    }
    else
    {
        ImageObject * obj = GetImageObjFromBuffer(line, row_, age_-1);
        if ( !obj )
            return NULL;

        return obj->GetImage();
    }
    
    return NULL;
}

// get the image from row_ or a newly created image which will have same width/heigth as orig_
wxImage* PixalcoApp::GetImageSameType(int row_, wxImage* orig_)
{
    if ( !orig_ )
        return NULL;
    
    // TODO: line should depend on line in which current algorithm runs
    int line = 0;
    
    ImageScope * scope = mMainFrame->GetImageScope(line, row_);
    if ( !scope )
        return NULL;
    
    if ( scope->GetImageObject() )
        mCopies[row_] = scope->GetImageObject()->Copy();

    if ( !IsSameImageType(orig_, scope->GetImage() ) )
    {
        if ( !scope->GetImageObject() )
            scope->SetImageObject( new ImageObject("copy") );   // TODO: make a useful name

        wxImage * newImage = new wxImage(orig_->GetWidth(), orig_->GetHeight());
        scope->GetImageObject()->SetImage( newImage );
    }

    return scope->GetImage();
}

bool PixalcoApp::IsSameImageType(wxImage* a_, wxImage* b_)
{
    if ( !a_ || !a_->Ok() || !b_ || !b_->Ok()
          ||    a_->GetWidth() != b_->GetWidth()
          ||    a_->GetHeight() != b_->GetHeight() )
    {
        return false;
    }

    return true;
}
        
void PixalcoApp::AddImageObjToMovieBuffer(ImageObject * imgObj_)
{
    if ( !imgObj_ )
        return;

//    cerr << imgObj_->GetDisplayName();

    imgObj_->AddRef();
    mMovieBuffer.push_front(imgObj_);
    if ( mMovieBuffer.size() > mMovieBufferSize )
    {
        mMovieBuffer.back()->Release();
        mMovieBuffer.pop_back();
    }     
}

wxImage* PixalcoApp::GetImageFromMovieBuffer(int age_)
{
    if ( mMovieBuffer.size() <= age_ )
        return NULL;

    list<ImageObject*>::iterator it=mMovieBuffer.begin();
    for ( int i=0; i < age_; ++it, ++i);
    
    return (*it)->GetImage();
}

void PixalcoApp::ClearBuffer(int line_, int row_)
{
    if ( row_ < 0 || row_ >= MAX_ROWS || line_ < 0 || line_ > 0 )
        return;
        
    while( mImageObjBuffer[row_]->size() > 0 )
    {
        mImageObjBuffer[row_]->back()->Release();
        mImageObjBuffer[row_]->pop_back();
    }
}

void PixalcoApp::ClearMovieBuffer()
{
    while ( mMovieBuffer.size() > 0 )
    {
        mMovieBuffer.front()->Release();
        mMovieBuffer.pop_front();
    }
}

void PixalcoApp::SetImageBufferSize(int size_)
{
    for ( int i=0; i<MAX_ROWS; ++i )
    {
        while( mImageObjBuffer[i]->size() > size_ )
        {
            mImageObjBuffer[i]->front()->Release();
            mImageObjBuffer[i]->pop_front();
        }
    }
    mImageBufferSize = size_;
}

void PixalcoApp::SetMovieBufferSize(int size_)
{
    while ( mMovieBuffer.size() > size_ )
    {
        mMovieBuffer.back()->Release();
        mMovieBuffer.pop_back();
    }
    mMovieBufferSize = size_;        
}


// just a test
void InMainCall(const char *text_)
{
    cerr << text_;
}

// A function for checking libdl errors 
int dlok(const char *msg)
{
    char *error;
    if ( error = dlerror() ) {
        printf("%s: %s\n", msg, error);
        return 0;
    }
    return 1;
}
void PixalcoApp::AddPlugin(const string& fileOnly_, const string& fileAndPath_)
{
/*    struct PixalcoPlugin plugin;
        
    // Load the library
    plugin.mHandle = dlopen(fileAndPath_.c_str(), RTLD_LAZY);
    cerr << "tried to load: " << fileAndPath_.c_str() << endl;
    dlok("loaderror");
    if ( plugin.mHandle )
    {
        cerr << "got the plugin" << endl;

        // Get the function address
        plugin.mInit = (InitPlugin)dlsym(plugin.mHandle, "PluginTest");
        if (!dlok("AddPlugin error:"))
            return;

        testCallMain * testCall = (testCallMain*)dlsym(plugin.mHandle, "testCall");
        if (!dlok("AddPlugin error:"))
            return;
        *testCall = InMainCall;

        plugin.mInit();        
} */
}
