#include "wx/wx.h"
#include <wx/listctrl.h>
#include "main.h"
#include "image_scope.h"
#include "main_frame.h"
#include "video.h"

BEGIN_EVENT_TABLE (MainFrame, wxFrame)
  EVT_MENU ( BASIC_EXIT,  MainFrame::OnExit)
  EVT_MENU ( BASIC_ABOUT, MainFrame::OnAbout)
  EVT_MENU ( BASIC_OPEN,  MainFrame::OnOpenFile)
  EVT_MENU ( PLUGIN_ADD,  MainFrame::OnPluginAdd)
  EVT_LIST_ITEM_ACTIVATED(STATIC_LIST_IMAGES, MainFrame::OnListImageActivated)
  EVT_LIST_ITEM_SELECTED(STATIC_LIST_IMAGES, MainFrame::OnListImageSelected)
  EVT_LIST_ITEM_ACTIVATED(STATIC_LIST_ALGORITHMS, MainFrame::OnListAlgorithmActivated)
  EVT_BUTTON(MOVIE_RESET_BTN, MainFrame::OnButtonMovieReset)
  EVT_BUTTON(MOVIE_RUN_BTN, MainFrame::OnButtonMovieRun)
  EVT_BUTTON(MOVIE_STEP_BTN, MainFrame::OnButtonMovieStep)
  EVT_COMMAND_SCROLL(MOVIE_POS_SCROLLER, MainFrame::OnScrollBarMoviePos)
END_EVENT_TABLE()

MainFrame::MainFrame (const wxChar *title,
                      int xpos, int ypos,
                      int width, int height)
    : wxFrame( (wxFrame *) NULL,
                -1,
                title,
                wxPoint(xpos, ypos),
                wxSize(width, height)
             )
{
    mMediaMode = MM_MOVIE;
    mMenuBar  = (wxMenuBar *) NULL;
    mFileMenu = (wxMenu *) NULL;
    mSettingsMenu = (wxMenu*) NULL;
    mListImages = (wxListCtrl *) NULL;
    mListAlgorithms = (wxListCtrl *) NULL;
    mImageScope = (ImageScope*)NULL;
    mImageScope2 = (ImageScope*)NULL;
    mImageScope3 = (ImageScope*)NULL;
    mTopSizer = (wxBoxSizer*)NULL;
    mListboxSizer = (wxBoxSizer*)NULL;
    mImageSizer = (wxBoxSizer*)NULL;
    mMovieCtrlSizer = NULL;
    mMovieResetBtn = NULL;
    mMovieRunBtn = NULL;
    mMovieStepBtn = NULL;
    mMoviePosScroller = NULL;
    mMovieButtonSizer = NULL;

    mFileMenu = new wxMenu;
	mFileMenu->Append(BASIC_OPEN,  wxT("&Open file"));
	mFileMenu->Append(BASIC_ABOUT, wxT("&About"));
    mFileMenu->AppendSeparator();
	mFileMenu->Append(BASIC_EXIT,  wxT("E&xit"));

    mSettingsMenu = new wxMenu;

    mPluginsMenu = new wxMenu;
	mPluginsMenu->Append(PLUGIN_ADD, wxT("&Add plugin"));
	mPluginsMenu->Append(PLUGIN_REMOVE, wxT("&Remove plugin..."));

    mMenuBar = new wxMenuBar;
	mMenuBar->Append(mFileMenu, wxT("&File"));
	mMenuBar->Append(mSettingsMenu, wxT("&Settings"));
	mMenuBar->Append(mPluginsMenu, wxT("&Plugins"));
    SetMenuBar(mMenuBar);

	mMovieResetBtn = new wxButton(this, MOVIE_RESET_BTN, wxT("Reset"), wxDefaultPosition );
	mMovieRunBtn = new wxButton(this, MOVIE_RUN_BTN, wxT("Run"), wxDefaultPosition );
	mMovieStepBtn = new wxButton(this, MOVIE_STEP_BTN, wxT("Step"), wxDefaultPosition );
    
    int range = 100;
    int thumbSize = 1;
    int pageSize = 10;
    int position = 0;
    mMoviePosScroller = new wxScrollBar(this, MOVIE_POS_SCROLLER, wxDefaultPosition, wxDefaultSize, wxSB_HORIZONTAL);
    mMoviePosScroller->SetScrollbar(position, thumbSize, range, pageSize, true);
    
    mListImages = new wxListCtrl( this, 
                                  STATIC_LIST_IMAGES,
                                  wxDefaultPosition, 
                                  wxSize(100, 100), 
                                  wxLC_SINGLE_SEL,      // style
                                  wxDefaultValidator, 
                                  wxT("listImages") );

    
    mListAlgorithms = new wxListCtrl( this, 
                                      STATIC_LIST_ALGORITHMS,
                                      wxPoint(100, 0), 
                                      wxSize(100, 100),
                                      wxLC_SINGLE_SEL,      // style
                                      wxDefaultValidator, 
                                      wxT("listAlgorithms") );
    
    mImageScope = new ImageScope(this, -1, NULL, wxPoint(0, 60), wxSize(200, 100), wxTAB_TRAVERSAL);
    mImageScope2 = new ImageScope(this, -1, NULL, wxPoint(201, 60), wxSize(200, 100), wxTAB_TRAVERSAL);
    mImageScope3 = new ImageScope(this, -1, NULL, wxPoint(402, 60), wxSize(200, 100), wxTAB_TRAVERSAL);
    
    mTopSizer = new wxBoxSizer( wxVERTICAL );        
    mListboxSizer = new wxBoxSizer( wxHORIZONTAL );
    mImageSizer = new wxBoxSizer( wxHORIZONTAL );
    mMovieCtrlSizer = new wxBoxSizer(wxVERTICAL);
    mMovieButtonSizer = new wxBoxSizer(wxHORIZONTAL);
    
    mMovieButtonSizer->Add(mMovieResetBtn, 1, wxEXPAND | wxALL, 5);
    mMovieButtonSizer->Add(mMovieRunBtn, 1, wxEXPAND | wxALL, 5);
    mMovieButtonSizer->Add(mMovieStepBtn, 1, wxEXPAND | wxALL, 5);
    
    mMovieCtrlSizer->Add(mMovieButtonSizer, 1, wxEXPAND | wxALL, 1);
    mMovieCtrlSizer->Add(mMoviePosScroller, 1, wxEXPAND | wxALL, 1);
    
    mListboxSizer->Add(mListImages,
                   1,            // make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                   wxALL,        //   and make border all around
                   5 );         // set border width to 10
    mListboxSizer->Add(mListAlgorithms,
                   1,            // don't make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                   wxALL,        //   and make border all around
                   5 );         // set border width to 10
    mImageSizer->Add(mImageScope,
                   1,            // make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                   wxALL,        //   and make border all around
                   5 );         // set border width to 10
    mImageSizer->Add(mImageScope2,
                   1,            // make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                   wxALL,        //   and make border all around
                   5 );         // set border width to 10
    mImageSizer->Add(mImageScope3,
                   1,            // make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                   wxALL,        //   and make border all around
                   5 );         // set border width to 10

    mTopSizer->Add(mListboxSizer,
                   0,            // don't make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                   wxALL,        //   and make border all around
                   5 );         // set border width to 10
    
    mTopSizer->Add( mMovieCtrlSizer, 0, wxEXPAND | wxALL, 5 );
    
    mTopSizer->Add(mImageSizer,
                   1,            // make vertically stretchable
                   wxEXPAND |    // make horizontally stretchable
                           wxALL,        //   and make border all around
                   5 );         // set border width to 10    

    SetSizer( mTopSizer );      // use the sizer for layout
    mTopSizer->SetSizeHints( this );
}

MainFrame::~MainFrame()
{
    delete mImageScope;
    delete mImageScope2;
    delete mImageScope3;
}

void MainFrame::OnOpenFile (wxCommandEvent & event_)
{
    static const wxChar *FILETYPES = _T(
            "All files|*.*|"
            "gif files|*.gif|"
            "tga files|*.tga|"
            "png files|*.png|"
            "bmp files|*.bmp|"
            "mpg files|*.mpg|"
            "avi files|*.avi|"
            "jpg files|*.jpg;*.jpeg|"            
           //"raw files|*.raw"
                                       );
    wxFileDialog fileDialog( this,
                             wxT("Open file"),
                             wxT(""),
                             wxT(""),
                             FILETYPES,
                             wxOPEN | wxCHANGE_DIR | wxMULTIPLE,
                             wxDefaultPosition);

    if (fileDialog.ShowModal() == wxID_OK)
    {
        wxArrayString filenames;
        wxArrayString paths;
        fileDialog.GetFilenames(filenames);
        fileDialog.GetPaths(paths);
        for ( int i=0; i < filenames.GetCount(); ++i )
        {
            // cerr << "file: " << (const char*)filenames[i] << "path:" << (const char*)paths[i] << endl;
	        PIXALCO.AddMediaFile((const char*)filenames[i].mb_str(), (const char*)paths[i].mb_str());
        }
    }
}

void MainFrame::OnPluginAdd(wxCommandEvent & event)
{
    static const wxChar *FILETYPES = _T(
            "plugins|*.so|"
             );
    wxFileDialog * openFileDialog = new wxFileDialog ( this,
	    	wxT("Open file"),
	    	wxT(""),
	    	wxT(""),
            FILETYPES,
            wxOPEN | wxCHANGE_DIR | wxMULTIPLE,
            wxDefaultPosition);

    if (openFileDialog->ShowModal() == wxID_OK)
    {
        wxArrayString filenames;
        wxArrayString paths;
        openFileDialog->GetFilenames(filenames);
        openFileDialog->GetPaths(paths);
        for ( int i=0; i < filenames.GetCount(); ++i )
        {
//            cerr << "file: " << (const char*)filenames[i] << "path:" << (const char*)paths[i] << endl;
	        PIXALCO.AddPlugin((const char*)filenames[i].mb_str(), (const char*)paths[i].mb_str());
        }
    }    
}

void MainFrame::OnAbout (wxCommandEvent & event_)
{
	wxString t = wxT("Pixalco");
    t.append( _T("\n\nUseful for easy testing of c++ image algorithms\n\nWritten by Michael Zeilfelder"));
  
	wxMessageDialog aboutDialog( this, t, wxT("About Pixalco"), wxOK);
    aboutDialog.ShowModal();
}

void MainFrame::OnExit (wxCommandEvent & event)
{
    Close(TRUE);
}

void MainFrame::UpdateMovieScrollerPos(Video * video_)
{
    if ( !video_ )
    {
        mMoviePosScroller->SetThumbPosition(0);
    }
    else
    {
        int position = (int)(100 * video_->GetPosition() / video_->GetDuration());
        mMoviePosScroller->SetThumbPosition(position);
    }
}

void MainFrame::OnListAlgorithmActivated(wxListEvent & event_)
{
	PIXALCO.ExecuteAlgorithm( (const char*)mListAlgorithms->GetItemText(event_.GetIndex()).mb_str() );
}

void MainFrame::OnListImageActivated(wxListEvent & event_)
{
	PIXALCO.SelectMediaFile((const char*)mListImages->GetItemText(event_.GetIndex()).mb_str());
}

void MainFrame::OnListImageDeleteSelected()
{
    long item = -1;
    for ( ;; )
    {
        item = mListImages->GetNextItem(item,
                                        wxLIST_NEXT_ALL,
                                        wxLIST_STATE_SELECTED);
        if ( item == -1 )
            break;
        
	    PIXALCO.RemoveMediaFile((const char*)mListImages->GetItemText(item).mb_str());
        
        break;  // single selection list, can only be one item
    }
}

void MainFrame::OnListImageSelected(wxListEvent & event_)
{
	MediaFile * media = PIXALCO.GetMediaFile((const char*)mListImages->GetItemText(event_.GetIndex()).mb_str());
    if ( !media )
        return;
    
    if ( media->GetMediaType() == MEDIA_MOVIE )
    {
        SetMediaMode(MM_MOVIE);
    }
    else
    {
        SetMediaMode(MM_BITMAP);
    }
}

void MainFrame::OnButtonMovieReset(wxCommandEvent & event_)
{
    int idImage = mListImages->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
    if ( idImage < 0 )
        return;
    
	MediaFile * media = PIXALCO.GetMediaFile((const char*)mListImages->GetItemText(idImage).mb_str());
    if ( media && media->GetMediaType() == MEDIA_MOVIE )
    {
	    PIXALCO.SeekMovie((const char*)mListImages->GetItemText(idImage).mb_str(), 0);
        mMoviePosScroller->SetThumbPosition(0);
    }
}

void MainFrame::OnButtonMovieRun(wxCommandEvent & event_)
{
    int idImage = mListImages->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
    if ( idImage < 0 )
        return;

    int idAlgorithm = mListAlgorithms->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

	PIXALCO.RunMovie( (const char*)mListImages->GetItemText(idImage).mb_str()
	                  ,(const char*)mListAlgorithms->GetItemText(idAlgorithm).mb_str() );
}

void MainFrame::OnButtonMovieStep(wxCommandEvent & event_)
{
    int idImage = mListImages->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
    if ( idImage < 0 )
        return;

    int idAlgorithm = mListAlgorithms->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

	PIXALCO.StepMovie( (const char*)mListImages->GetItemText(idImage).mb_str()
	                   ,(const char*)mListAlgorithms->GetItemText(idAlgorithm).mb_str() );
}

void MainFrame::OnScrollBarMoviePos(wxScrollEvent & event_)
{
    int idImage = mListImages->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
    if ( idImage < 0 )
        return;
    
	MediaFile * media = PIXALCO.GetMediaFile((const char*)mListImages->GetItemText(idImage).mb_str());
    if ( media && media->GetMediaType() == MEDIA_MOVIE )
    {
        long pos = (media->mVideo->GetDuration() / 100) * event_.GetPosition();
	    PIXALCO.SeekMovie((const char*)mListImages->GetItemText(idImage).mb_str(), pos);
    }
}

int MainFrame::GetNrImageLines() const
{
    // TODO: support several lines
    return 1;
}

// TODO: support variable rows, support several lines
int MainFrame::GetNrRowsForLine(int line_) const
{
    if ( line_ != 1 )
        return 0;

    return 3;
}

// TODO: support variable rows, support several lines
ImageScope* MainFrame::GetImageScope(int line_, int row_) const
{
    if ( line_ > GetNrImageLines() )
        return NULL;
    
    switch (row_)
    {
        case 0:
            return mImageScope;
        case 1:
            return mImageScope2;
        case 2:
            return mImageScope3;
    }
    
    return NULL;
}

void MainFrame::SetMediaMode(MediaMode mode_)
{
    if ( mMediaMode != mode_ )
    {
        if ( mode_ == MM_MOVIE )
            mTopSizer->Show(mMovieCtrlSizer, true );
        else if ( mode_ == MM_BITMAP )
            mTopSizer->Show(mMovieCtrlSizer, false );
        mTopSizer->Layout();
        mMediaMode = mode_;
    }
}
