#ifndef MAIN_FRAME_H
#define MAIN_FRAME_H

enum EVENT_ID
{ 
    BASIC_EXIT    =   wxID_HIGHEST + 1,
    BASIC_OPEN ,
    BASIC_ABOUT,

    PLUGIN_ADD,
    PLUGIN_REMOVE,
    
    STATIC_LIST_IMAGES,
    STATIC_LIST_ALGORITHMS,
    
    MOVIE_RESET_BTN,
    MOVIE_RUN_BTN,
    MOVIE_STEP_BTN,
    MOVIE_POS_SCROLLER,
};

class ImageScope;
class Video;
class wxListCtrl;
class wxMenuBar;
class wxMenu;

enum MediaMode
{
    MM_BITMAP,
    MM_MOVIE,
};

class MainFrame : public wxFrame
{
    public:
        MainFrame( const wxChar *title, 
                   int xpos, int ypos, 
                   int width, int height);
        ~MainFrame();
    
        void OnOpenFile (wxCommandEvent & event);
        void OnAbout    (wxCommandEvent & event);
        void OnExit     (wxCommandEvent & event);
        void OnPluginAdd(wxCommandEvent & event);
        void OnListImageActivated(wxListEvent & event);
        void OnListImageSelected(wxListEvent & event);
        void OnListImageDeleteSelected();
        void OnListAlgorithmActivated(wxListEvent & event);
        void OnButtonMovieReset(wxCommandEvent & event_);
		void OnButtonMovieRun(wxCommandEvent & event_);
		void OnButtonMovieStep(wxCommandEvent & event_);
        void OnScrollBarMoviePos(wxScrollEvent & event_);
                
        // TODO: this is no longer just an imageList - it keeps also movienames now
        wxListCtrl& AccessImagesList()      { return *mListImages; }
        wxListCtrl& AccessAlgorithmsList()  { return *mListAlgorithms; }

        int GetNrImageLines() const;
        int GetNrRowsForLine(int line_) const;
        ImageScope* GetImageScope(int line_, int row_) const;
        void UpdateMovieScrollerPos(Video * video_);

        DECLARE_EVENT_TABLE()

    protected:
        void SetMediaMode(MediaMode mode_);
        MediaMode GetMediaMode() const  { return mMediaMode; }

    private:
        wxMenuBar   *mMenuBar;
        wxMenu      *mFileMenu;
        wxMenu      *mSettingsMenu;
        wxMenu      *mPluginsMenu;
        wxListCtrl  *mListImages;
        wxListCtrl  *mListAlgorithms;
        ImageScope  *mImageScope;
        ImageScope  *mImageScope2;  // TODO: just that way for prototype
        ImageScope  *mImageScope3;
        wxBoxSizer  *mTopSizer;
        wxBoxSizer  *mListboxSizer;
        wxBoxSizer  *mImageSizer;
        
        wxBoxSizer  *mMovieCtrlSizer;
        wxBoxSizer  *mMovieButtonSizer;
        wxButton    *mMovieResetBtn;
        wxButton    *mMovieRunBtn;
        wxButton    *mMovieStepBtn;
        wxScrollBar *mMoviePosScroller;

        MediaMode   mMediaMode;
};

#endif // MAIN_FRAME_H
